#!/bin/bash

#==============================================================================
# perl
#==============================================================================

fun_perl_sourced=1

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

[[ -n $fun_arch_sourced ]] || . $(dirname $BASH_SOURCE)/fun-arch.sh
export EPAGES_KERNEL

[[ -n $fun_epages_sourced ]] || . $(dirname $BASH_SOURCE)/fun-epages.sh
export etc_default

[[ -n $fun_linux_sourced ]] || . $(dirname $BASH_SOURCE)/fun-linux.sh
export backup_file

[[ -n $fun_pkg_sourced ]] || . $(dirname $BASH_SOURCE)/fun-pkg.sh
export pkg_installed

#------------------------------------------------------------------------------
# provide
#------------------------------------------------------------------------------

export backup_obsolete_perl set_perl_tree what_needs_lib

#------------------------------------------------------------------------------
# define
# -----------------------------------------------------------------------------

#..............................................................................
# backup_obsolete_perl
#..............................................................................
backup_obsolete_perl() {
  local USAGE='
syntax: backup_obsolete_perl
description: find files that existed in the old RPM, but not in the new RPM.
  if found, those files will be packed into a .tgz file.
returns: name of file with list of found files (nothing if no file found)
'
  [[ $# -eq 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  local EPAGES=${EPAGES:-"${RPM_INSTALL_PREFIX:-/srv/epages}"/eproot}
  local EPAGES_PERL=${EPAGES_PERL:-"$EPAGES"/Perl}

  local PKG_VERSION=$(pkg_installed -V epages-perl | tail -1)
  [[ -n $PKG_VERSION ]] || return

  local PERL_PKG=epages-perl-$PKG_VERSION

  if [[ $EPAGES_KERNEL != ${EPAGES_KERNEL%d} ]] ; then
    # debian
    dpkg-query -L epages-perl
  else
    # redhat
    rpm -q $PERL_PKG --list
  fi | sort > /tmp/$PERL_PKG.rpmlist

  BIN_EX='bin/((ep)?node|karma.sh|npm|optipng|jsl)'
  LINUX_EX='lib/linux/(CORE/config.h|Config.pm|Config_heavy.pl)'
  SITE_EX='lib/site_perl/ppm.xml'

  find "$EPAGES_PERL"/{*,.??*}/ -type f 2>/dev/null \
  | egrep -v "/($BIN_EX|$LINUX_EX|$SITE_EX)" | sort > /tmp/$PERL_PKG.installed

  [[ -s /tmp/$PERL_PKG.installed ]] || return

  comm -13 /tmp/$PERL_PKG.rpmlist /tmp/$PERL_PKG.installed > /tmp/$PERL_PKG.norpm 2>/dev/null
  [[ -s /tmp/$PERL_PKG.norpm ]] || return

  cat /tmp/$PERL_PKG.norpm | tar zcfT - - > "$EPAGES"/$PERL_PKG-norpm.tgz 2>/dev/null
  echo /tmp/$PERL_PKG.norpm
}

#..............................................................................
# what_needs_lib
#..............................................................................
what_needs_lib() {
  local USAGE='
syntax: what_needs_lib FILE LIB1 ...
description: FILE contains a file list (as e.g. from backup_obsolete_perl).
  for each file of that list show files that use at least one of the given LIBs.
returns: list of files which use one of the given LIBs
example: what_needs_lib /tmp/$PERL_PKG.norpm libssl.so.1.0.0 libcrypto.so.1.0.0
'
  [[ $# -lt 2 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local FILELIST=$1 ; shift
  local LIBRARIES=$@

  [[ -s $FILELIST ]] || return

  while read path ; do
    [[ -f "$path" ]] || continue
    file -L "$path" | egrep -q 'ELF [0-9]+-bit' || continue
    objdump -p "$path" | grep NEEDED | egrep -q " (${LIBRARIES// /|})$" && echo "$path"
  done < $FILELIST
}

#..............................................................................
# set_perl_tree
#..............................................................................
set_perl_tree () {
  local USAGE='
syntax: set_perl_tree
description: sets path to epages-perl in perl files
'
  [[ $# -eq 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  local EPAGES=${EPAGES:-"${RPM_INSTALL_PREFIX:-/srv/epages}"/eproot}
  local EPAGES_PERL=${EPAGES_PERL:-"$EPAGES"/Perl}

  pushd "$EPAGES_PERL"/lib/linux >/dev/null || return 1

  for i in Config.pm Config_heavy.pl ../site_perl/ppm.xml CORE/config.h ; do
    backup_file $i.ept
    sed "s,=EPAGES_PERL=,$EPAGES_PERL,g" $i.ept > $i
  done

  if grep -q ' Perl/' perllocal.pod 2>/dev/null ; then
    cp -p perllocal.pod perllocal.pod.tmp
    sed "s, Perl/, $EPAGES_PERL/,g" perllocal.pod.tmp > perllocal.pod
    rm -f perllocal.pod.tmp
  fi

  popd >/dev/null
  pushd "$EPAGES_PERL"/lib >/dev/null

  for i in $(find . -name .packlist -exec grep \-l '^Perl/' {} \;) ; do
    cp -p $i $i.tmp
    sed "s,^Perl/,$EPAGES_PERL/,g" $i.tmp > $i
    rm -f $i.tmp
  done
  popd >/dev/null
}

#------------------------------------------------------------------------------
# end
# -----------------------------------------------------------------------------
