#!/bin/bash

#==============================================================================
# epages-perl.spec
#==============================================================================

export TERM=

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

. $(dirname $0)/fun-epages.sh
export update_epages_environment epages_service

. $(dirname $0)/fun-pkg.sh
export pkg_instance

. $(dirname $0)/fun-perl.sh
export backup_obsolete_perl what_needs_lib

. $(dirname $0)/fun-java.sh
export test_version

#------------------------------------------------------------------------------
# define
# -----------------------------------------------------------------------------

# INSTANCE is always:
# last:   0 - remove last version of package
# first:  1 - install the first time
# update: 2 (or higher) - update (depending on the number of versions installed)

#..............................................................................
# epages-perl-pre
#..............................................................................
epages-perl-pre() {
  if [[ $# -ne 2 || -z $1 || -z $2 ]] ; then
    echo 'syntax: epages-perl-pre INSTANCE RPM_VERSION'
    echo 'description: pre-uninstallation tasks of epages-perl package'
    echo 'example: epages-perl-pre 0 6.16.4'
    return 1
  fi
  local INSTANCE=$(pkg_instance $1)
  local VERSION=$2

  [[ $INSTANCE = first ]] || rm -rf /srv/epages/eproot/Perl/man/man8

  [[ -n $EPAGES_SKIP_SPEC ]] && return 0

  # epages Flex AS are stopped in epages-perl.spec
  [[ -x /etc/init.d/ePagesConfigServer ]] || epages_service stop AS

  if [[ $INSTANCE != first ]] ; then
    # backup obsolete perl files
    PERL_BACKUP=$(backup_obsolete_perl)
    [[ -s $PERL_BACKUP ]] && ln -s $PERL_BACKUP /tmp/epages-perl-$VERSION.obsoleted
  fi

  return 0
}

#..............................................................................
# epages-perl-post
#..............................................................................
epages-perl-post() {
  if [[ $# -ne 2 || -z $1 || -z $2 ]] ; then
    echo 'syntax: epages-perl-post INSTANCE RPM_VERSION'
    echo 'description: post-installation tasks of epages-perl package'
    echo 'example: epages-perl-post 1 6.16.4'
    return 1
  fi
  local INSTANCE=$(pkg_instance $1)
  local VERSION=$2

  [[ -n $EPAGES_SKIP_SPEC ]] && return 0

  update_epages_environment -i "$1"

  # epages Flex AS are started in epages-perl.spec
  [[ -x /etc/init.d/ePagesConfigServer ]] || epages_service start AS

  [[ $INSTANCE = first ]] && return 0

  # if epages-perl-pre backed up obsolete perl files
  # then check these files for loading libssl/crypto

  # return if no backup file list found
  local BACKUP_FILE_LIST=/tmp/epages-perl-$VERSION.obsoleted
  [[ -s $BACKUP_FILE_LIST ]] || return 0

  local OLD_VERSION=$(readlink -f $BACKUP_FILE_LIST)
  OLD_VERSION=${OLD_VERSION##*/epages-perl-}
  OLD_VERSION=${OLD_VERSION%.norpm}

  # return if outdated version exists and is >= 6.17.36
  [[ -z ${OLD_VERSION//[0-9.]/} ]] && test_version $OLD_VERSION -ge 6.17.36 && return 0

  # shorten file list by files existing in current version
  OPENSSL_EX='Crypt/OpenSSL/(Bignum/Bignum|DSA/DSA|RSA/RSA|Random/Random|X509/X509).so'
  OTHER_EX='((Crypt|Net)/SSLeay/SSLeay|DBD/mysql/mysql).so'
  egrep -v "/($OPENSSL_EX|$OTHER_EX|lib32/)" $BACKUP_FILE_LIST > $BACKUP_FILE_LIST.short

  # find binaries which load libssl/crypto (1.0.0)
  what_needs_lib $BACKUP_FILE_LIST.short libssl.so.1.0.0 libcrypto.so.1.0.0 | grep .

  # return if no such binaries found
  [[ $? = 0 ]] || return 0

  # die if outdated binaries found
  echo -e "\nError: additional CPAN modules found that must be recompiled"
  echo "  because OpenSSL has changed."
  echo -e "Without re-compilation ePages won't work correctly.\n"
  return 1
}

#..............................................................................
# epages-perl-preun
#..............................................................................
epages-perl-preun() {
  if [[ $# -ne 2 || -z $1 || -z $2 ]] ; then
    echo 'syntax: epages-perl-preun INSTANCE RPM_VERSION'
    echo 'description: preun-uninstallation tasks of epages-perl package'
    echo 'example: epages-perl-preun 0 6.16.4'
    return 1
  fi
  local INSTANCE=$(pkg_instance $1)
  local VERSION=$2

  [[ -n $EPAGES_SKIP_SPEC ]] && return 0

  # epages Flex AS are stopped in epages-perl.spec
  [[ -x /etc/init.d/ePagesConfigServer ]] || epages_service stop AS

  return 0
}

#..............................................................................
# epages-perl-postun
#..............................................................................
epages-perl-postun() {
  if [[ $# -ne 2 || -z $1 || -z $2 ]] ; then
    echo 'syntax: epages-perl-postun INSTANCE RPM_VERSION'
    echo 'description: postun-uninstallation tasks of epages-perl package'
    echo 'example: epages-perl-postun 0 6.16.4'
    return 1
  fi
  local INSTANCE=$(pkg_instance $1)
  local VERSION=$2

  [[ -n $EPAGES_SKIP_SPEC ]] && return 0

  # epages Flex AS are started in epages-perl.spec
  [[ $INSTANCE = last || -x /etc/init.d/ePagesConfigServer ]] || epages_service start AS

  return 0
}

#------------------------------------------------------------------------------
# execute
# -----------------------------------------------------------------------------

case $1 in
  pre)     epages-perl-pre    $2 $3 ; exit $? ;;
  post)    epages-perl-post   $2 $3 ; exit $? ;;
  preun)   epages-perl-preun  $2 $3 ; exit $? ;;
  postun)  epages-perl-postun $2 $3 ; exit $? ;;
  *) exit 1 ;;
esac

#------------------------------------------------------------------------------
# exit
# -----------------------------------------------------------------------------
