#!/bin/bash

#==============================================================================
# epages-update.sh
#==============================================================================

if [[ ! -w /etc/passwd ]] ; then
  echo "Error: Only 'root' can execute this program!"
  exit 1
fi

LANG=C # use base language for standard output of some commands
LC_ALL=C

#------------------------------------------------------------------------------
# arguments
#------------------------------------------------------------------------------

usage() {
  [[ -n $1 ]] && echo -e "Error:\n  $@"
  echo "Usage:"
  echo "  $0 {--singlehost|--distributed} [--version V] [--repo R]"
  echo "Options:"
  echo "  --version : target version for ePages patch (default: latest version)"
  echo "  --repo    : ePages repository (default: http://epages.com/repo/epages)"
  echo "  --singlehost : unpack and install patch on single host installation"
  echo "  --distributed : just unpack patch on distributed ePages installation"
  echo "Examples:"
  echo "  $0 --singlehost"
  echo "  $0 --singlehost --version 6.17.34 --repo ftp://epages-software.de/repo/epages"
  echo "  $0 --distributed --version 6.17.34"
  exit 1
}

TEMP=$(getopt -o '' --long singlehost,distributed,version:,repo: -n $0 -- "$@") || usage
eval set -- "$TEMP"

unset VERSION REPO SINGLEHOST DISTRIBUTED
while true ; do
  case "$1" in
    --singlehost) SINGLEHOST=1; export EPAGES_DO_PATCH=1; shift ;;
    --distributed) DISTRIBUTED=1; export EPAGES_DO_PATCH=0; shift ;;
    --version) [[ -z $2 ]] && usage ; VERSION=$2 ; shift 2 ;;
    --repo) [[ -z $2 ]] && usage ; REPO=$2 ; shift 2 ;;
    --) shift ; break ;;
    *) echo $@ ; usage ;;
  esac
done
# no other arguments allowed
[[ -n $1 ]] && usage

if [[ ( -n $SINGLEHOST && -n $DISTRIBUTED ) || -z "$SINGLEHOST$DISTRIBUTED" ]] ; then
  usage "$0: choose either --singlehost or --distributed"
fi

#-----------------------------------------------------------------------
# ECHO
#-----------------------------------------------------------------------

ECHO () {
  local LINE_NUMBER=$(printf "%3d" $1)
  shift
  echo -e "[${0##*/}:$LINE_NUMBER] $@"
}

#-----------------------------------------------------------------------
# find OS
#-----------------------------------------------------------------------

unset OS
if [[ -f /etc/debian_version ]] ; then
  ECHO $LINENO "Debian found: /etc/debian_version -> $(cat /etc/debian_version)"
  OS=debian
elif [[ -f /etc/redhat-release ]] ; then
  ECHO $LINENO "Redhat found: /etc/redhat-release -> $(cat /etc/redhat-release)"
  OS=redhat
else
  echo "$0: operating system not supported: $(uname -a)"
  exit 1
fi

#-----------------------------------------------------------------------
# update epages-release
#-----------------------------------------------------------------------

REPO_URL=${REPO:-http://epages.com/repo/epages}

unset OLD_PKG_VERSION NEW_PKG_VERSION
if [[ $OS = debian ]] ; then
  OLD_PKG_VERSION=$(dpkg-query --show -f '${Version}\n' epages-release)
  ECHO $LINENO curl -O $REPO_URL/apt/epages-release_all.deb
  curl -O $REPO_URL/apt/epages-release_all.deb
  ECHO $LINENO dpkg -i epages-release_all.deb
  dpkg -i epages-release_all.deb
  NEW_PKG_VERSION=$(dpkg-query --show -f '${Version}\n' epages-release)
else
  ECHO $LINENO rpm -hUv $REPO_URL/yum/epages-release.noarch.rpm
  rpm -hUv $REPO_URL/yum/epages-release.noarch.rpm
fi

if [[ -n "$REPO$VERSION" ]] ; then
  VERSION_ARG=${VERSION:+-b $VERSION}
  REPO_ARG=${REPO:+-r $REPO_URL}
  ECHO $LINENO /var/epages/SetRepository.sh $VERSION_ARG $REPO_ARG
  /var/epages/SetRepository.sh $VERSION_ARG $REPO_ARG
elif [[ $OS = debian && $OLD_PKG_VERSION != $NEW_PKG_VERSION ]] ; then
  ECHO $LINENO apt-get -y update
  apt-get -y update
fi

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

. /etc/default/epages6

INSTALL_PACKAGE=/var/epages/install-RPM.sh

#-----------------------------------------------------------------------
# update or install packages from epages-setup
#-----------------------------------------------------------------------

ECHO $LINENO ENABLE_REPO=epages-setup $INSTALL_PACKAGE --update epages-perl epages-nodejs
ENABLE_REPO=epages-setup $INSTALL_PACKAGE --update epages-perl epages-nodejs || exit 1

#-----------------------------------------------------------------------
# update packages from epages-base
#-----------------------------------------------------------------------

ECHO $LINENO ENABLE_REPO=epages-base $INSTALL_PACKAGE --update-only epages-requires epages-webconf
ENABLE_REPO=epages-base $INSTALL_PACKAGE --update-only epages-requires epages-webconf || exit 1

#-----------------------------------------------------------------------
# update packages from epages-update
#-----------------------------------------------------------------------

if [[ $OS = debian ]] ; then
  ECHO $LINENO apt-get -y --only-upgrade install 'epages-[^p]'
  apt-get -y --only-upgrade install 'epages-[^p]' || exit 1
else
  ECHO $LINENO yum -y update 'epages-[a-oq-z]*' --disablerepo epages-setup --enablerepo epages-update
  yum -y update 'epages-[a-oq-z]*' --disablerepo epages-setup --enablerepo epages-update || exit 1
fi

#-----------------------------------------------------------------------
# update patch package
#-----------------------------------------------------------------------

if [[ $OS = debian ]] ; then
  ECHO $LINENO EPAGES_DO_PATCH=$EPAGES_DO_PATCH apt-get -y install epages-patch
  apt-get -y install epages-patch || exit 1
else
  ECHO $LINENO EPAGES_DO_PATCH=$EPAGES_DO_PATCH yum -y update epages-patch --disablerepo epages-setup --enablerepo epages-update
  yum -y update epages-patch --disablerepo epages-setup --enablerepo epages-update || exit 1
fi

if [[ -n $DISTRIBUTED ]] ; then
  echo
  echo "Now go on with the steps explained in:"
  echo "http://helpcenter.epages.com/Doc/current/epages/Manual/en/ePages_Patch_Installation.html#Patch_a_Distributed_ePages_Installation"
  echo
fi

#-----------------------------------------------------------------------
# exit
#-----------------------------------------------------------------------

exit 0
